==> ginmappeR_0.99.9.tar.gz: ginmappeR/R/KEGGFunctions.R <==
source("R/utilsFunctions.R")

############################
# KEGG database to UniProt #
############################

.getKEGG2UniProt <- function(keggId, exhaustiveMapping = FALSE){

    tryCatch(
        {
            .checkKEGGIdExists(keggId)

            query <- keggConv('uniprot', keggId)
            if(identical(query, character(0))|identical(query, "")){
                return(character(0))
            }else{
                splittedQuery <- strsplit(query,':')
                splittedQuery <- unname(unlist(splittedQuery)[2*(1:length(query))])
                if(!exhaustiveMapping){
                    splittedQuery <- splittedQuery[[1]]
                }
                return(splittedQuery)
            }
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

###################################
# KEGG database to NCBI databases #
###################################

# Direct translation through keggConv
.getKEGG2NCBIDT <- function(keggId){

    query <- keggConv('ncbi-geneid', keggId)
    query <- c(query, keggConv('ncbi-proteinid', keggId))

    splittedQuery <- strsplit(query,':')
    splittedQuery <- unname(unlist(splittedQuery)[2*(1:length(query))])

    aux <- keggGet(keggId)
    if('DBLINKS' %in% names(aux[[1]])){
        aux <- unlist(lapply(strsplit(aux[[1]]$DBLINKS[grep("^NCBI", aux[[1]]$DBLINKS)], ":"), function(x) trimws(x[2])))
        if(is.null(aux)){aux <- character(0)}
        splittedQuery <- unique(c(splittedQuery, aux))
    }

    if(identical(splittedQuery, character(0))|is.null(splittedQuery)|identical(splittedQuery, "")){
        return(character(0))
    }else{
        return(splittedQuery)
    }
}


# Through UniProt method: translate first to UniProt and use
# getUniProt2NCBIxxx methods to get to NCBIxxx
.getKEGG2NCBITUP <- function(keggId, ncbiDB, exhaustiveMapping = FALSE, bySimilarGenes = TRUE){

    result <- list()
    upIDs <- suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getKEGG2UniProt(keggId)))

    if(length(upIDs)>0){

        for(upId in upIDs){
            ncbiIDs <- switch (ncbiDB,
                 'protein' = suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getUniProt2NCBIProtein(upId, exhaustiveMapping = exhaustiveMapping, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes))),
                 'nucleotide' = suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getUniProt2NCBINucleotide(upId, exhaustiveMapping = exhaustiveMapping, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes))),
                 'gene' = suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getUniProt2NCBIGene(upId, exhaustiveMapping = exhaustiveMapping, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes)))
            )

            if(length(ncbiIDs)>0){
                result <- .mergeNamedLists(result, ncbiIDs)
            }
        }
    }
    return(lapply(result, unique))
}

.getKEGG2NCBI <- function(keggId, ncbiDB, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkKEGGIdExists(keggId)

    result <- list()
    translationsDT <- .getKEGG2NCBIDT(keggId)
    translationsDT <- translationsDT[which(sapply(translationsDT, .checkIdInNCBIDataBase, ncbiDB=ncbiDB))]

    # First, try to retrieve a direct translation
    if(!identical(translationsDT, character(0))){
        if(!exhaustiveMapping){
            if(!detailedMapping){
                return(translationsDT[1])
            }else{
                result[['DT']] <- translationsDT[1]
            }
            return(result)
        }else{
            result[['DT']] <- translationsDT
        }
    }

    # Then, try to translate through UniProt database
    result <- .mergeNamedLists(result, .getKEGG2NCBITUP(keggId, ncbiDB, exhaustiveMapping = exhaustiveMapping,
                                    bySimilarGenes = bySimilarGenes))

    if(!detailedMapping){
        result <- unique(unlist(result, recursive = FALSE, use.names = FALSE))
        if(is.null(result)){result <- character(0)}
    }else{
        result <- lapply(result, unique)
    }
    return(result)
}

.getKEGG2NCBIProtein <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')
    tryCatch(
        {
            return(.getKEGG2NCBI(keggId, 'protein', exhaustiveMapping = exhaustiveMapping, detailedMapping = detailedMapping,
                                 bySimilarGenes = bySimilarGenes))
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

.getKEGG2NCBINucleotide <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')
    tryCatch(
        {
            return(.getKEGG2NCBI(keggId, 'nucleotide', exhaustiveMapping = exhaustiveMapping, detailedMapping = detailedMapping,
                                 bySimilarGenes = bySimilarGenes))
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

.getKEGG2NCBIGene <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')
    tryCatch(
        {
            return(.getKEGG2NCBI(keggId, 'gene', exhaustiveMapping = exhaustiveMapping, detailedMapping = detailedMapping,
                                 bySimilarGenes = bySimilarGenes))
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

#########################
# KEGG database to CARD #
#########################

# Auxiliar function to retrieve intermediate NCBI ids and check their translations
# to CARD on the go
.getKEGG2CARDexhaustiveMappingAux <- function(keggId, ncbiDB, bySimilarGenes){
    aroIndex <- .loadAROIndex()

    # This function translates a bunch of NCBI ids to CARD
    .auxNCBI2CARD <- function(translations, ncbiDB){
        if(identical(ncbiDB, 'protein')){
            translations <- lapply(translations, FUN = function(x) unlist(sapply(x, USE.NAMES = FALSE, FUN = function(auxId){
                auxId <- strsplit(auxId,'.', fixed = TRUE)[[1]][[1]]
                return(aroIndex[gsub("\\..*", "", aroIndex$Protein.Accession) == auxId, "ARO.Accession"])
            })))
        }else{
            translations <- lapply(translations, FUN = function(x) unlist(sapply(x, USE.NAMES = FALSE, FUN = function(auxId){
                auxId <- strsplit(auxId,'.', fixed = TRUE)[[1]][[1]]
                return(aroIndex[gsub("\\..*", "", aroIndex$DNA.Accession) == auxId, "ARO.Accession"])
            })))
        }
        return(translations)
    }

    # Direct translation
    translationsDT <- .getKEGG2NCBIDT(keggId)
    if(!identical(translationsDT, character(0))){
        translationsDT <- lapply(.auxNCBI2CARD(list('DT' = translationsDT),ncbiDB), unique)
        translationsDT <- translationsDT[lengths(translationsDT) > 0]
        if(length(translationsDT[['DT']]) > 0){
            return(translationsDT)
        }
    }

    # Through UniProt
    translationsTUP <- .getKEGG2NCBITUP(keggId, ncbiDB, exhaustiveMapping = TRUE, bySimilarGenes = bySimilarGenes)
    if(length(translationsTUP)>0){
        translationsTUP <- lapply(.auxNCBI2CARD(translationsTUP, ncbiDB), unique)
        translationsTUP <- translationsTUP[lengths(translationsTUP) > 0]
        if(length(translationsTUP)>0){
            return(translationsTUP)
        }else{
            return(list())
        }
    }
}


# Added bySimilarGenes param to deactivate it in case it takes long time to finish, or activate it if no
# translation is found
.getKEGG2CARD <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')

    tryCatch(
        {
            .checkKEGGIdExists(keggId)

            aroIndex <- .loadAROIndex()
            result <- list()

            # Handle not exhaustiveMapping case, trying to get the fastest result present in CARD
            if(!exhaustiveMapping){
                proteinId <-.getKEGG2CARDexhaustiveMappingAux(keggId, 'protein', bySimilarGenes)
                if(length(proteinId)>0){
                    proteinId <- Filter(Negate(is.null), proteinId[c("DT", "1.0", "0.9", "0.5")])
                    result[names(proteinId)[1]] <- proteinId[[1]][[1]]
                }else{
                    nucleotideId <-.getKEGG2CARDexhaustiveMappingAux(keggId, 'nucleotide', bySimilarGenes)
                    if(length(nucleotideId)>0){
                        nucleotideId <- Filter(Negate(is.null), nucleotideId[c("DT", "1.0", "0.9", "0.5")])
                        result[names(nucleotideId)[1]] <- nucleotideId[[1]][[1]]
                    }
                }
            }else{
                # Handle exhaustiveMapping case, trying to retrieve all possible cases
                proteinId <- suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getKEGG2NCBIProtein(keggId, exhaustiveMapping = TRUE, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes)))
                if(length(proteinId)>0){
                    result <- lapply(proteinId, FUN = function(x) unlist(sapply(x, USE.NAMES = FALSE, FUN = function(auxId){
                        auxId <- strsplit(auxId,'.', fixed = TRUE)[[1]][[1]]
                        return(aroIndex[gsub("\\..*", "", aroIndex$Protein.Accession) == auxId, "ARO.Accession"])
                    })))
                    result <- result[lengths(result) > 0]
                }

                nucleotideId <- suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getKEGG2NCBINucleotide(keggId, exhaustiveMapping = TRUE, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes)))
                if(length(nucleotideId)>0){
                    nucleotideId <- lapply(nucleotideId, FUN = function(x) unlist(sapply(x, USE.NAMES = FALSE, FUN = function(auxId){
                        auxId <- strsplit(auxId,'.', fixed = TRUE)[[1]][[1]]
                        return(aroIndex[gsub("\\..*", "", aroIndex$DNA.Accession) == auxId, "ARO.Accession"])
                    })))
                }
                nucleotideId <- nucleotideId[lengths(nucleotideId) > 0]
                result <- .mergeNamedLists(result, nucleotideId)
            }

            # Finally, parse the output according to detailedMapping parameter
            if(!detailedMapping){
                result <- unique(unlist(result, recursive = FALSE, use.names = FALSE))
                if(is.null(result)){result <- character(0)}
            }else{
                result <- lapply(result, unique)
                result <- result[lengths(result) > 0]
                if(length(result)==0){result <- list()}
            }
            return(result)
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

########################
# Functions interfaces #
########################

getKEGG2UniProt <- function(keggId, exhaustiveMapping = FALSE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE) {
                return(.retryHandler(f, keggId, exhaustiveMapping))})
        })(.getKEGG2UniProt), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping)))
    })(keggId, exhaustiveMapping))
}

getKEGG2NCBIProtein <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2NCBIProtein), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}

getKEGG2NCBINucleotide <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2NCBINucleotide), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}

getKEGG2NCBIGene <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2NCBIGene), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}

getKEGG2CARD <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    .checkIfCARDIsDownloaded()
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2CARD), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}

==> ginmappeR_0.99.10.tar.gz: ginmappeR/R/KEGGFunctions.R <==
source("R/utilsFunctions.R")

############################
# KEGG database to UniProt #
############################

.getKEGG2UniProt <- function(keggId, exhaustiveMapping = FALSE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')

    tryCatch(
        {
            .checkKEGGIdExists(keggId)

            query <- keggConv('uniprot', keggId)
            if(identical(query, character(0))|identical(query, "")){
                return(character(0))
            }else{
                splittedQuery <- strsplit(query,':')
                splittedQuery <- unname(unlist(splittedQuery)[2*(1:length(query))])
                if(!exhaustiveMapping){
                    splittedQuery <- splittedQuery[[1]]
                }
                return(splittedQuery)
            }
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

###################################
# KEGG database to NCBI databases #
###################################

# Direct translation through keggConv
.getKEGG2NCBIDT <- function(keggId){

    query <- keggConv('ncbi-geneid', keggId)
    query <- c(query, keggConv('ncbi-proteinid', keggId))

    splittedQuery <- strsplit(query,':')
    splittedQuery <- unname(unlist(splittedQuery)[2*(1:length(query))])

    aux <- keggGet(keggId)
    if('DBLINKS' %in% names(aux[[1]])){
        aux <- unlist(lapply(strsplit(aux[[1]]$DBLINKS[grep("^NCBI", aux[[1]]$DBLINKS)], ":"), function(x) trimws(x[2])))
        if(is.null(aux)){aux <- character(0)}
        splittedQuery <- unique(c(splittedQuery, aux))
    }

    if(identical(splittedQuery, character(0))|is.null(splittedQuery)|identical(splittedQuery, "")){
        return(character(0))
    }else{
        return(splittedQuery)
    }
}


# Through UniProt method: translate first to UniProt and use
# getUniProt2NCBIxxx methods to get to NCBIxxx
.getKEGG2NCBITUP <- function(keggId, ncbiDB, exhaustiveMapping = FALSE, bySimilarGenes = TRUE){

    result <- list()
    upIDs <- suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getKEGG2UniProt(keggId)))

    if(length(upIDs)>0){

        for(upId in upIDs){
            ncbiIDs <- switch (ncbiDB,
                 'protein' = suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getUniProt2NCBIProtein(upId, exhaustiveMapping = exhaustiveMapping, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes))),
                 'nucleotide' = suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getUniProt2NCBINucleotide(upId, exhaustiveMapping = exhaustiveMapping, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes))),
                 'gene' = suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getUniProt2NCBIGene(upId, exhaustiveMapping = exhaustiveMapping, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes)))
            )

            if(length(ncbiIDs)>0){
                result <- .mergeNamedLists(result, ncbiIDs)
            }
        }
    }
    return(lapply(result, unique))
}

.getKEGG2NCBI <- function(keggId, ncbiDB, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkKEGGIdExists(keggId)

    result <- list()
    translationsDT <- .getKEGG2NCBIDT(keggId)
    translationsDT <- translationsDT[which(sapply(translationsDT, .checkIdInNCBIDataBase, ncbiDB=ncbiDB))]

    # First, try to retrieve a direct translation
    if(!identical(translationsDT, character(0))){
        if(!exhaustiveMapping){
            if(!detailedMapping){
                return(translationsDT[1])
            }else{
                result[['DT']] <- translationsDT[1]
            }
            return(result)
        }else{
            result[['DT']] <- translationsDT
        }
    }

    # Then, try to translate through UniProt database
    result <- .mergeNamedLists(result, .getKEGG2NCBITUP(keggId, ncbiDB, exhaustiveMapping = exhaustiveMapping,
                                    bySimilarGenes = bySimilarGenes))

    if(!detailedMapping){
        result <- unique(unlist(result, recursive = FALSE, use.names = FALSE))
        if(is.null(result)){result <- character(0)}
    }else{
        result <- lapply(result, unique)
    }
    return(result)
}

.getKEGG2NCBIProtein <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')
    tryCatch(
        {
            return(.getKEGG2NCBI(keggId, 'protein', exhaustiveMapping = exhaustiveMapping, detailedMapping = detailedMapping,
                                 bySimilarGenes = bySimilarGenes))
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

.getKEGG2NCBINucleotide <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')
    tryCatch(
        {
            return(.getKEGG2NCBI(keggId, 'nucleotide', exhaustiveMapping = exhaustiveMapping, detailedMapping = detailedMapping,
                                 bySimilarGenes = bySimilarGenes))
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

.getKEGG2NCBIGene <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')
    tryCatch(
        {
            return(.getKEGG2NCBI(keggId, 'gene', exhaustiveMapping = exhaustiveMapping, detailedMapping = detailedMapping,
                                 bySimilarGenes = bySimilarGenes))
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

#########################
# KEGG database to CARD #
#########################

# Auxiliar function to retrieve intermediate NCBI ids and check their translations
# to CARD on the go
.getKEGG2CARDexhaustiveMappingAux <- function(keggId, ncbiDB, bySimilarGenes){
    aroIndex <- .loadAROIndex()

    # Direct translation
    translationsDT <- .getKEGG2NCBIDT(keggId)
    if(!identical(translationsDT, character(0))){
        translationsDT <- lapply(.auxNCBI2CARD(list('DT' = translationsDT),ncbiDB, aroIndex), unique)
        translationsDT <- translationsDT[lengths(translationsDT) > 0]
        if(length(translationsDT[['DT']]) > 0){
            return(translationsDT)
        }
    }

    # Through UniProt
    translationsTUP <- .getKEGG2NCBITUP(keggId, ncbiDB, exhaustiveMapping = TRUE, bySimilarGenes = bySimilarGenes)
    if(length(translationsTUP)>0){
        translationsTUP <- lapply(.auxNCBI2CARD(translationsTUP, ncbiDB, aroIndex), unique)
        translationsTUP <- translationsTUP[lengths(translationsTUP) > 0]
        if(length(translationsTUP)>0){
            return(translationsTUP)
        }else{
            return(list())
        }
    }
}


# Added bySimilarGenes param to deactivate it in case it takes long time to finish, or activate it if no
# translation is found
.getKEGG2CARD <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE){
    .checkBoolean(exhaustiveMapping, 'exhaustiveMapping')
    .checkBoolean(detailedMapping, 'detailedMapping')
    .checkBoolean(bySimilarGenes, 'bySimilarGenes')

    tryCatch(
        {
            .checkKEGGIdExists(keggId)

            aroIndex <- .loadAROIndex()
            result <- list()

            # Handle not exhaustiveMapping case, trying to get the fastest result present in CARD
            if(!exhaustiveMapping){
                proteinId <-.getKEGG2CARDexhaustiveMappingAux(keggId, 'protein', bySimilarGenes)
                if(length(proteinId)>0){
                    proteinId <- Filter(Negate(is.null), proteinId[c("DT", "1.0", "0.9", "0.5")])
                    result[names(proteinId)[1]] <- proteinId[[1]][[1]]
                }else{
                    nucleotideId <-.getKEGG2CARDexhaustiveMappingAux(keggId, 'nucleotide', bySimilarGenes)
                    if(length(nucleotideId)>0){
                        nucleotideId <- Filter(Negate(is.null), nucleotideId[c("DT", "1.0", "0.9", "0.5")])
                        result[names(nucleotideId)[1]] <- nucleotideId[[1]][[1]]
                    }
                }
            }else{
                # Handle exhaustiveMapping case, trying to retrieve all possible cases
                proteinId <- suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getKEGG2NCBIProtein(keggId, exhaustiveMapping = TRUE, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes)))
                if(length(proteinId)>0){
                    result <- lapply(proteinId, FUN = function(x) unlist(sapply(x, USE.NAMES = FALSE, FUN = function(auxId){
                        auxId <- strsplit(auxId,'.', fixed = TRUE)[[1]][[1]]
                        return(aroIndex[gsub("\\..*", "", aroIndex$Protein.Accession) == auxId, "ARO.Accession"])
                    })))
                    result <- result[lengths(result) > 0]
                }

                nucleotideId <- suppressMessages(.cacheErrorHandler(raiseError=TRUE, .getKEGG2NCBINucleotide(keggId, exhaustiveMapping = TRUE, detailedMapping = TRUE, bySimilarGenes = bySimilarGenes)))
                if(length(nucleotideId)>0){
                    nucleotideId <- lapply(nucleotideId, FUN = function(x) unlist(sapply(x, USE.NAMES = FALSE, FUN = function(auxId){
                        auxId <- strsplit(auxId,'.', fixed = TRUE)[[1]][[1]]
                        return(aroIndex[gsub("\\..*", "", aroIndex$DNA.Accession) == auxId, "ARO.Accession"])
                    })))
                }
                nucleotideId <- nucleotideId[lengths(nucleotideId) > 0]
                result <- .mergeNamedLists(result, nucleotideId)
            }

            # Finally, parse the output according to detailedMapping parameter
            if(!detailedMapping){
                result <- unique(unlist(result, recursive = FALSE, use.names = FALSE))
                if(is.null(result)){result <- character(0)}
            }else{
                result <- lapply(result, unique)
                result <- result[lengths(result) > 0]
                if(length(result)==0){result <- list()}
            }
            return(result)
        },
        error = function(e) {return(.errorMessageHandler(e))},
        warning = function(e) {return(.warningMessageHandler(e))}
    )
}

########################
# Functions interfaces #
########################

getKEGG2UniProt <- function(keggId, exhaustiveMapping = FALSE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE) {
                return(.retryHandler(f, keggId, exhaustiveMapping))})
        })(.getKEGG2UniProt), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping)))
    })(keggId, exhaustiveMapping))
}

getKEGG2NCBIProtein <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2NCBIProtein), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}

getKEGG2NCBINucleotide <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2NCBINucleotide), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}

getKEGG2NCBIGene <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2NCBIGene), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}

getKEGG2CARD <- function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
    .checkIfCARDIsDownloaded()
    vectorizedFunc <- Vectorize(
        (function(f){
            return(function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
                return(.retryHandler(f, keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))})
        })(.getKEGG2CARD), vectorize.args = c('keggId'), USE.NAMES = FALSE, SIMPLIFY = FALSE)
    return((function(keggId, exhaustiveMapping = FALSE, detailedMapping = FALSE, bySimilarGenes = TRUE) {
        return(.resultParser(keggId, exhaustiveMapping, vectorizedFunc(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes)))
    })(keggId, exhaustiveMapping, detailedMapping, bySimilarGenes))
}
